#include <stdio.h>
#include <string.h>
#include "generalmod.h"

int main(int argc, char *argv[])
{
  long i, j, k;
  Hypoplasti_unsat_expansive_thermal huet;
  long nparams=huet.nparms;    //number of parameters
  long ngstrain=huet.ngstrain; //number of generalised strain tensor components (8 for actual implementation, see constructor in generalmod.h)
  long ngstress=huet.ngstress; //number of generalised stress tensor components (7 for actual implementation, see constructor in generalmod.h)
  long nstatev=huet.nstatev;   //number of state variables (9 for actual implementation, see constructor in generalmod.h)
  double params[nparams];
  double eps[ngstrain];
  double sig[ngstress];
  double statev[nstatev];
  double deps[ngstrain];
  double dsig[ngstress];
  double dstatev[nstatev];
  double epsn[ngstrain];
  double sign[ngstress];
  double statevn[nstatev];
  double sn, tds, tdeps, ideps, ids;
  FILE *out;

  if (argc != 2){
    fprintf(stderr, "\n Wrong number of command line arguments\n");
    fprintf(stderr, " Use: %s output_file_name\n", argv[0]);
    return 1;
  }
  else{
    out = fopen(argv[1], "wt");
    if (out == NULL){
      fprintf(stderr, "\nCannot open output file '%s'\n", argv[1]);
      return 1;
    }
  }
  params[0] = 25.0;  // phideg
  params[1] = 0.13;  // lam_star
  params[2] = 0.06;  // kap_star
  params[3] = 1.73;  // n_star
  params[4] = 0.25;  // nu
  params[5] = 0.012; // ns
  params[6] = -0.005;// ls
  params[7] = -0.07; // nt
  params[8] = 0.0;   // lt
  params[9] = 1.0;   // m
  params[10] = 0.00015; // alpha_s
  params[11] = 0.07;    // kap_m
  params[12] = -2000.0; // sm_star
  params[13] = 0.45;    // em_star
  params[14] = 0.002;   // csh
  params[15] = -2700.0; // se_ref
  params[16] = 0.5;     // em_ref
  params[17] = 294.0;   // tref
  params[18] = 0.118;   // at
  params[19] = -1.54e-4;// bt
  params[20] = 1.0;     // aer
  params[21] = 0.7;     // lambdap0
  params[22] = 1.0;     // p_t
  
  memset(eps, 0, sizeof(eps));
  memset(sig, 0, sizeof(sig));
  memset(statev, 0, sizeof(statev));
  memset(deps, 0, sizeof(deps));
  memset(dsig, 0, sizeof(dsig));
  memset(dstatev, 0, sizeof(dstatev));
  // set initial stress to eigenstress value
  sig[0] = sig[1] = sig[2] = -0.1;
  // set initial values of void ratio
  statev[0] = 0.64659;
  // set initial value of ascan
  statev[7] = 0.0;
  // set initial value of suction
  statev[1] = eps[6] = -1.0e5;
  // set inital value of temperature
  statev[3] = eps[7] =  294.0;
  
  // initialization of the model 
  huet.soil_model(eps, sig, statev, deps, NULL, dstatev, NULL, params, 0, 0);
  // set computed initial Sr
  statev[2] = sig[6];

  // initial strain increment
  ideps = -3.4e-6;
  // initial suction increment
  ids = -99999.989467157+1.0e+5;
  // total range of strain increment which will be tested
  tdeps = -3.5e-6 - ideps;
  // total range of suction increment which will be tested
  tds = 0.0 - ids;
  // set initial increment of strains
  deps[1] = ideps;
  // set suction increment
  deps[6] = ids;
  for (j=0; j<1; j++){
    deps[6] = ids;
    /*    if (j == 1)
      deps[6] = -99979.7388325997 + 1.0e+5; 
    if (j == 2)
      deps[6] = 6.56; 
    if (j == 3)
    deps[6] = -99999.8443879852 + 1.0e5;*/
    for (i=0; i<1000; i++){
      deps[1] = ideps + tdeps / 1000 * i;
      huet.soil_model(eps, sig, statev, deps, dsig, dstatev, NULL, params, 2, 0);
      for (k=0; k<ngstress; k++)
        sign[k] = sig[k] + dsig[k];
      for (k=0; k<ngstrain; k++)
        epsn[k] = eps[k] + deps[k];
      for (k=0; k<nstatev; k++)
        statevn[k] = statev[k] + dstatev[k];
      sn = eps[6] + deps[6];
   
      fprintf(out, "%le %le %le", epsn[1], sn, sign[1]);
      /*    
        fprintf(out, "\nepsn:\n");
        for (i=0; i<ngstrain; i++)
          fprintf(out, "%le ", epsn[i]);
        fprintf(out, "\nsign:\n");
        for (i=0; i<ngstress; i++)
          fprintf(out, "%le ", sign[i]);
        fprintf(out, "\nstatevn:\n");
        for (i=0; i<nstatev; i++)
          fprintf(out, "%le ", statevn[i]);
      */
      fprintf(out, "\n");
    }
  }
  //  fprintf(out, "--------------------------\n");
  fclose(out);
  return 0;
}
