#ifndef NORMMAT_H
#define NORMMAT_H

#include "vector.h"
#include "matrix.h"
#include "iotools.h"

/**
   class contains material based on Eurocode
   
   components of array other
   
   
*/
class normmat
{
 public:
  
  normmat (void);
  ~normmat (void);
  
  void read (XFILE *in);
  
  
  //modul pruznosti betonu v tlaku prislusny 0,6 fc v kPa
  void compute_ecnom ();
  
  //pretvoreni oceli pro stanoveni modulu pruznosti e0cn
  void compute_epsuk ();
  
  //modul pruznosti odpovidajici zplastizovani oceli v kPa
  void compute_e0cn ();
  
  //soucinitel pro definovani pracovniho diagramu betonu
  void compute_k ();
  
  //plocha prurezu
  void compute_a ();
  
  //ucinna vyska prurezu
  void compute_d ();
  
  //pevnost betonu v tlaku uvazovana pro vypocet
  void compute_fc ();

  //svisla koncova sila ve stycniku i
  void compute_z1 (long ipp,double lp);

  //momentova koncova sila ve stycniku i
  void compute_z2 (long ipp,double lp);

  //vodorovna pomocna koncova sila ve stycniku i
  void compute_z3p (long ipp,double lp);

  //vodorovna koncova sila ve stycniku i
  void compute_z3 ();
  
  //vyztuz pro tahovou oblast od ucinku ohyboveho momentu as1
  double compute_as1 (double lp);
  
  //vyztuz pro tlakovou oblast od ucinku ohyboveho momentu as2
  double compute_as2 (double lp);
  
  //ohybovy moment na dannem elementu v polovine intervalu
  void compute_m (double lp);
  
  //absolutni hodnota ohyboveho momentu na dannem elementu v polovine intervalu
  void compute_mpp (double lp);
  
  //normalova sila na dannem elementu v polovine intervalu
  double compute_n0 (double lp);

  //pocet iteraci
  long compute_iter ();
  
  //funkce ktera rusi vliv betonove casti prurezu
  void compute_tcv ();
  
  //VLIV NORMALOVE SILY
  
  //pretvoreni od ucinku normalove sily
  double compute_epsn ();
  
  //maximalni tlakova normalova sila, kterou lze zatizit prurez
  double compute_nmax ();
  
  //prvni clen pro linearizaci
  double compute_an1 ();
  
  //druhy clen pro linearizaci
  double compute_an2 ();
  
  //prace na betonove casti prurezu pro prubeh normalove sily (-z3 + fn x); pres element delky lp 
  double compute_in1 (double lp);
  
  //prace na ocelove casti prurezu pro prubeh normalove sily (-z3 + fn x); pres element delky lp 
  double compute_in2 (double lp);
  
  //celkova prace na prurezu pro prubeh normalove sily (-z3 + fn x); pres element delky lp 
  double compute_kinh ();
  
  //pomocna prace na prurezu pro prubeh normalove sily (-z3 + fn x); pres element delky lp 
  double compute_kind (double lp);
  
  //pomer prace vnitrnich sil a pomocne prace vnitrnich sil na dannem elementu od ucinku normalove sily
  double compute_kin ();
  
  //TLACENA VYZTUZ + NORMALOVA SILA + x = h
  
  //pretvoreni betonu, kdy tlacena plocha x=h
  double compute_epschr1 ();
  
  //hodnota ohyboveho momentu, kdy tlacena plocha x=h a nedochazi k teceni oceli As1 a As2
  double compute_mhr1 ();
  
  //TLACENA VYZTUZ + NORMALOVA SILA + TECENI V As1 + x < h
  
  //pretvoreni betonu, kdy dochazi k teceni vyztuze As1, ale netece As2 a ani nedochazi k drceni betonu
  double compute_epschr2 ();
  
  //hodnota ohyboveho momentu, kdy dochazi k teceni vyztuze As1, ale netece As2 a ani nedochazi k drceni betonu 
  double compute_mhr2 ();
  
  //TLACENA VYZTUZ + NORMALOVA SILA + TECENI V As2 + x < h
  
  //pretvoreni betonu, kdy dochazi k teceni vyztuze As2, ale netece As1 a ani nedochazi k drceni betonu, x < h
  double compute_epschr3 ();
  
  //hodnota ohyboveho momentu, kdy dochazi k teceni vyztuze As2, ale netece As1 a ani nedochazi k drceni betonu, x < h
  double compute_mhr3 ();
  
  //TLACENA VYZTUZ + NORMALOVA SILA + TECENI V As2 + x > h
  
  //pretvoreni betonu, kdy dochazi k teceni vyztuze As2, ale netece As1 a ani nedochazi k drceni betonu, x > h
  double compute_epschr4 ();
  
  //maximalni pretvoreni betonu od ucinku ohyboveho momentu 
  double compute_epscmax1 ();
  
  //pretvoreni betonu, kdy uz tece vyztuz As2, netece As1 a ani nedochazi k drceni betonu, ale dochazi ke zmene tlacene plochy x = h 
  double compute_epschr4b ();
  
  //pomocne pretvoreni betonu
  double compute_epschrk1a ();
  
  //hodnota ohyboveho momentu, kdy dochazi k teceni vyztuze As2, ale netece As1 a ani nedochazi k drceni betonu, x > h
  double compute_mhr4 ();
  
  //hodnota ohyboveho momentu, kdy uz tece vyztuz As2, netece As1 a ani nedochazi k drceni betonu, ale dochazi ke zmene tlacene plochy x = h
  double compute_mhr4b ();
  
  //pretvoreni betonu, ktere odpovida max. dosazenemu ohybovemu momentu, kdy uz tece vyztuz As2, netece As1 a dochazi k drceni betonu, x > h
  double compute_epscmax1p ();
  
  //max. hodnota ohyboveho momentu, kdy uz tece vyztuz As2, netece As1 a dochazi k drceni betonu, x > h
  double compute_mmax1a ();
  
  //hranicni ohybovy moment
  double compute_mh1 ();
  
  //hranicni pretvoreni betonu
  double compute_epsch1 ();
  
  //hodnota ohyboveho momentu pro pretvoreni na polovine danneho intervalu, x > h
  double compute_mh1hrk1a ();
  
  //pomocna hodnota ohyboveho momentu
  double compute_mhrk1a ();
  
  //hranicni ohybovy moment
  double compute_mh3 ();
  
  //hranicni pretvoreni betonu
  double compute_epsch3 ();
  
  //hranicni ohybovy moment
  double compute_mh2 ();
  
  //hranicni pretvoreni betonu
  double compute_epsch2 ();
  
  //pomocne pretvoreni betonu
  double compute_epscder2 ();
  
  //hodnota maximalniho ohyboveho momentu, kdy uz tece vyztuz As2, netece As1 a dochazi k drceni betonu, x > h
  double compute_mmax1 ();
  
  //pomocna hodnota ohyboveho momentu
  double compute_mhrk1 ();
  
  //pomocne pretvoreni betonu
  double compute_epschrk1 ();
  
  //hodnota ohyboveho momentu pro pretvoreni na polovine danneho intervalu, x > h 
  double compute_mh1hrk1 ();
  
  //TLACENA VYZTUZ + NORMALOVA SILA + TECENI V As1 + TECENI V As2 + x < h
  
  //PRVNI TEKLO As1
  
  //pretvoreni betonu, kdy dochazi k teceni vyztuze As2, uz teklo As1 a nedochazi k drceni betonu, x < h
  double compute_epschr5 ();
  
  //pretvoreni betonu po dosazeni mezniho pretvoreni oceli As1, kdy jeste nedoslo k teceni vyztuze As2 a nedochazi k drceni betonu, x < h 
  double compute_epscs11 ();
  
  //hodnota ohyboveho momentu, kdy dochazi k teceni vyztuze As2, uz teklo As1 a nedochazi k drceni betonu, x < h
  double compute_mhr5 ();
  
  //hodnota ohyboveho momentu po dosazeni mezniho pretvoreni betonu v tlaku, tece As1 a jeste nedoslo k teceni vyztuze As2
  double compute_mmax2 ();
  
  //hodnota ohyboveho momentu po dosazeni mezniho pretvoreni oceli As1 pri jejim tazeni, kdy jeste nedoslo k teceni vyztuze As2 a nedochazi k drceni betonu, x < h
  double compute_ms11 ();
  
  //pomocna hodnota ohyboveho momentu
  double compute_mh5a ();
  
  //pomocne pretvoreni betonu
  double compute_epsch5a ();
  
  //pomocny hranicni ohybovy moment
  double compute_mh5p ();
  
  //pomocne hranicni pretvoreni betonu
  double compute_epsch5p ();
  
  //hodnota ohyboveho momentu pro pretvoreni na polovine danneho intervalu, x < h
  double compute_mh25a ();
  
  //mezni pretvoren betonu v tlaku po dosazeni mezni nosnosti pruezu, kter se bliz meznmu pretvoreni oceli As1 pri jejim tazeni, kdy jeste nedoslo k teceni vyztuze As2 a nedochazi k drceni betonu, x < h
  double compute_epscder4 ();
  
  //hodnota ohyboveho momentu po dosazeni mezni nosnosti pruezu, kter se bliz meznmu pretvoreni oceli As1 pri jejim tazeni, kdy jeste nedoslo k teceni vyztuze As2 a nedochazi k drceni betonu, x < h
  double compute_mmax5 ();
  
  //hranicni ohybovy moment
  double compute_mh5 ();
  
  //hranicni pretvoreni betonu
  double compute_epsch5 ();
  
  //PRVNI TEKLO As2
  
  //pomocne pretvoreni betonu
  double compute_epscp ();
  
  //pretvoreni betonu v tlaku, kdy dochazi k teceni vyztuze As1, uz teklo As2 a nedochazi k drceni betonu, x < h
  double compute_epschr6 ();
  
  //hodnota ohyboveho momentu, kdy dochazi k teceni vyztuze As1, uz teklo As2 a nedochazi k drceni betonu, x < h
  double compute_mhr6 ();
  
  //hodnota ohyboveho momentu po dosazeni mezniho pretvoreni betonu v tlaku, tece As2 a jeste nedoslo k teceni vyztuze As1, x < h
  double compute_mmax3 ();
  
  //pomocna hodnota ohyboveho momentu
  double compute_mh6a (); 
  
  //pomocna hodnota pretvoreni betonu v tlaku
  double compute_epsch6a (); 
  
  //hranicni ohybovy moment
  double compute_mh7 ();  
  
  //hranicni pretvoreni betonu v tlaku
  double compute_epsch7 ();  
  
  //pretvoreni betonu v tlaku na polovine danneho intervalu
  double compute_epsch12 ();
  
  //pretvoren vyztuze As1, po dosazeni pretvoren betonu v tlaku epsch1
  double compute_epss1h1 ();
  
  //pretvoren vyztuze As1, po dosazeni pretvoren betonu v tlaku epsch2
  double compute_epss1h2 ();
  
  //hodnota ohyboveho momentu pro pretvoreni na polovine danneho intervalu
  double compute_mh01 ();
  
  //hodnota ohyboveho momentu pro pretvoreni na polovine danneho intervalu
  double compute_mh12 ();
  
  //hodnota ohyboveho momentu pro pretvoreni na polovine danneho intervalu
  double compute_mh25 ();
  
  //hodnota ohyboveho momentu pro pretvoreni na polovine danneho intervalu
  double compute_mh36a ();
  
  //pomocne pretvoreni betonu
  double compute_epscder1 ();
  
  //hranicni ohybovy moment
  double compute_mh6 (); 
  
  //hranicni pretvoreni betonu v tlaku
  double compute_epsch6 (); 
  
  //hodnota ohyboveho momentu pro pretvoreni na polovine danneho intervalu
  double compute_mh36 ();
  
  //maximalni ohybovy moment, tece As2 a jeste nedoslo k teceni vyztuze As1 a k dosazeni mezniho pretvoreni betonu, x < h
  double compute_mmax4 ();
  
  //hodnota ohyboveho momentu pro pretvoreni na polovine danneho intervalu
  double compute_mh7maxa ();
  
  //maximaln mozny ohybovy moment dosazeny na dannem elementu pri danne normalove sile, bez zohlednen epscder3
  double compute_mhmaxa ();
  
  //pomocna hodnota pretvoreni betonu v tlaku
  double compute_epscder3 ();
  
  //maximaln mozny ohybovy moment dosazeny na dannem elementu pri danne normalove sile, pokud epscder3 lez v dannm definicnm oboru
  double compute_mhmax ();
  
  //pomocna hodnota ohyboveho momentu
  double compute_mh8p (); 
  
  //pomocna hodnota pretvoreni betonu v tlaku
  double compute_epsch8p (); 
  
  //pomocna hodnota ohyboveho momentu
  double compute_mh8 (); 
  
  //pomocna hodnota pretvoreni betonu v tlaku
  double compute_epsch8 (); 
  
  //hodnota ohyboveho momentu pro pretvoreni na polovine danneho intervalu
  double compute_mh78 ();
  
  //ohybovy moment pri kterem uz nespolupusobi beton a dochazi k teceni vyztuze As2, netece As1
  double compute_mhoc1 ();
  
  //ohybovy moment pri kterem uz nespolupusobi beton a dochazi k meznimu teceni vyztuze As2, netece As1
  double compute_mmaxoc1 ();
  
  //pretvoreni oceli As1 pri kterem uz nespolupusobi beton a dochazi k meznimu teceni vyztuze As2, netece As1
  double compute_epss1oc1 ();
  
  //ohybovy moment pri kterem uz nespolupusobi beton a dochazi k teceni vyztuze As1 a uz teklo As2
  double compute_mhoc2 ();
  
  //pretvoreni oceli As1 pri kterem uz nespolupusobi beton a dochazi k teceni vyztuze As1 a uz teklo As2
  double compute_epss1oc2 ();
  
  //ohybovy moment pri kterem uz nespolupusobi beton a dochazi k meznimu teceni vyztuze As1 a prekroceni mezniho pretvoreni As2
  double compute_mmaxoc2 ();
  
  //hodnota ohyboveho momentu pro prvni bod zlomu na pracovnim diagramu prurezu
  double compute_mht1 (); 
  
  //hodnota ohyboveho momentu pro druhy bod zlomu na pracovnim diagramu prurezu
  double compute_mht2 (); 
  
  //hodnota ohyboveho momentu pro treti bod zlomu na pracovnim diagramu prurezu
  double compute_mht3 ();
  
  //hodnota ohyboveho momentu pro ctvrty bod zlomu na pracovnim diagramu prurezu
  double compute_mht4 ();
  
  //maximalni hodnota ohyboveho momentu na pracovnim diagramu prurezu
  double compute_mmax ();
  
  //hodnota pretvoreni betonu v tlaku pro prvni bod zlomu na pracovnim diagramu prurezu
  double compute_epscht1 ();
  
  //hodnota pretvoreni betonu v tlaku pro druhy bod zlomu na pracovnim diagramu prurezu
  double compute_epscht2 ();
  
  //hodnota pretvoreni betonu v tlaku pro treti bod zlomu na pracovnim diagramu prurezu
  double compute_epscht3 ();
  
  //hodnota pretvoreni betonu v tlaku pro ctvrty bod zlomu na pracovnim diagramu prurezu
  double compute_epscht4 ();
  
  //maximalni hodnota pretvoreni betonu v tlaku na pracovnim diagramu prurezu
  double compute_epscmax ();
  
  //absolutni uvazovana hodnota ohyboveho momentu na dannem elementu v polovine intervalu
  void compute_mp ();
  
  //funkce ktera rusi vliv betonove casti prurezu
  void compute_tcn ();
  
  //hodnota pretvoreni betonu v tlaku odpovidajici i-temu vnitrnimu uzlu
  void compute_epscsp ();
  
  //hodnota pretvoreni oceli As1 v tahu, od ucinku ohyboveho momentu, odpovidajici i-temu vnitrnimu uzlu
  void compute_epss1sp ();
  
  //hodnota prvni konstanty pro vypocet ohyboveho momentu v zavislosti na pretvoreni betonu v tlaku, odpovidajici i-temu vnitrnimu uzlu
  void compute_a1sp ();
  
  //hodnota druhe pomocne konstanty pro vypocet ohyboveho momentu v zavislosti na pretvoreni betonu v tlaku, odpovidajici i-temu vnitrnimu uzlu
  void compute_a2spp ();
  
  //hodnota druhe konstanty pro vypocet ohyboveho momentu v zavislosti na pretvoreni betonu v tlaku, odpovidajici i-temu vnitrnimu uzlu
  void compute_a2sp ();
  
  //hodnota prvni konstanty pro vypocet pretvoreni oceli As1 v tahu, od ucinku ohyboveho momentu, odpovidajici i-temu vnitrnimu uzlu
  void compute_b1sp ();
  
  //hodnota druhe pomocne konstanty pro vypocet pretvoreni oceli As1 v tahu, od ucinku ohyboveho momentu, odpovidajici i-temu vnitrnimu uzlu
  void compute_b2spp ();
  
  //hodnota druhe konstanty pro vypocet ohyboveho momntu v zavislosti na pretvoreni betonu v tlaku, odpovidajici i-temu vnitrnimu uzlu
  void compute_b2sp ();
  
  //hodnota ohyboveho momentu pro vypocet prvni hodnoty funkce prace vnitrnich sil,odpovidajici i-temu vnitrnimu uzlu
  void compute_mr ();
  
  //hodnota pretvoreni betonu v tlaku pro vypocet prvni hodnoty funkce prace vnitrnich sil, od ucinku ohyboveho momentu, odpovidajici i-temu vnitrnimu uzlu
  void compute_epscr ();
  
  //hodnota pretvoreni oceli As1 v tahu pro vypocet prvni hodnoty funkce prace vnitrnich sil, od ucinku ohyboveho momentu, odpovidajici i-temu vnitrnimu uzlu
  void compute_epss1r ();
  
  //hodnota pretvoreni oceli As1 v tahu pro vypocet prvni hodnoty funkce pomocne prace vnitrnich sil, od ucinku ohyboveho momentu, odpovidajici i-temu vnitrnimu uzlu
  void compute_epss1spp ();
  
  //velikost pomocne tlacene plochy prurezu
  void compute_xr ();
  
  //velikost tlacene plochy prurezu
  void compute_xsp ();
  
  //prvni integracni hodnota udavajici pocatek funkce prace vnitrnich sil pripadajici betonu, od ucinku ohyboveho momentu, odpovidajici i-temu vnitrnimu uzlu
  void compute_ir1 ();
  
  //druha integracni hodnota funkce prace vnitrnich sil pripadajici betonu, od ucinku ohyboveho momentu,odpovidajici i-temu vnitrnimu uzlu
  void compute_ims1 ();
  
  //prvni integracni hodnota udavajici pocatek funkce prace vnitrnich sil pripadajici tazene oceli As1, od ucinku ohyboveho momentu, odpovidajici i-temu vnitrnimu uzlu
  void compute_ir2 ();
  
  //druha integracni hodnota funkce prace vnitrnich sil pripadajici tazene oceli As1, od ucinku ohyboveho momentu,odpovidajici i-temu vnitrnimu uzlu
  void compute_ims2 ();
  
  //prvni integracni hodnota udavajici pocatek funkce prace vnitrnich sil pripadajici tlacene oceli As2, od ucinku ohyboveho momentu, odpovidajici i-temu vnitrnimu uzlu
  void compute_ir3 ();
  
  //druha integracni hodnota prace vnitrnich sil pripadajici tlacene oceli As2, od ucinku ohyboveho momentu,odpovidajici i-temu vnitrnimu uzlu
  void compute_ims3 ();
  
  //prvni integracni hodnota udavajici pocatek funkce prace vnitrnich sil, od ucinku ohyboveho momentu, odpovidajici i-temu vnitrnimu uzlu
  void compute_ic1 ();
  
  //druha integracni hodnota prace vnitrnich sil, od ucinku ohyboveho momentu, odpovidajici i-temu vnitrnimu uzlu
  void compute_ic2 ();
  
  //pomocna integracni hodnota prace vnitrnich sil, od ucinku ohyboveho momentu, odpovidajici i-temu vnitrnimu uzlu
  void compute_icp ();
  
  //prvni clen integracni funkce prace vnitrnich sil, od ucinku ohyboveho momentu, odpovidajici i-temu vnitrnimu uzlu
  void compute_c1 ();
  
  //druhy clen integracni funkce prace vnitrnich sil, od ucinku ohyboveho momentu, odpovidajici i-temu vnitrnimu uzlu
  void compute_c2 ();
  
  //prvni clen pomocne integracni funkce prace vnitrnich sil, od ucinku ohyboveho momentu, odpovidajici i-temu vnitrnimu uzlu
  void compute_c1p ();
  
  //druhy clen pomocne integracni funkce prace vnitrnich sil, od ucinku ohyboveho momentu, odpovidajici i-temu vnitrnimu uzlu
  void compute_c2p ();
  
  //integral prace vnitrnich sil pres dany element
  void compute_kih (double lp);
  
  //integral pomocne prace vnitrnich sil pres dany element
  void compute_kid (double lp);
  
  //pomer prace vnitrnich sil a pomocne prace vnitrnich sil na dannem elementu
  void compute_ki ();
  
  //clen matice tuhosti
  double compute_k11 (double lp);
  
  //clen matice tuhosti
  double compute_k12 (double lp);

  //clen matice tuhosti
  double compute_k13 (double lp);
  
  //clen matice tuhosti
  double compute_k14 (double lp);
  
  //zatizeni od ucinku rovnomerneho spojiteho zatizeni (prava strana);
  double compute_f1 (double lp);
  
  //clen matice tuhosti
  double compute_k21 (double lp);
  
  //clen matice tuhosti
  double compute_k22 (double lp);

  //clen matice tuhosti
  double compute_k23 (double lp);
  
  //clen matice tuhosti
  double compute_k24 (double lp);
  
  //zatizeni od ucinku rovnomerneho spojiteho zatizeni (prava strana);
  double compute_f2 (double lp);
  
  //clen matice tuhosti
  double compute_k31 (double lp);
  
  //clen matice tuhosti
  double compute_k32 (double lp);
  
  //clen matice tuhosti
  double compute_k33 (double lp);
  
  //clen matice tuhosti
  double compute_k34 (double lp);
  
  //zatizeni od ucinku rovnomerneho spojiteho zatizeni (prava strana);
  double compute_f3 (double lp);
  
  //clen matice tuhosti
  double compute_k41 (double lp);
  
  //clen matice tuhosti
  double compute_k42 (double lp);
  
  //clen matice tuhosti
  double compute_k43 (double lp);
  
  //clen matice tuhosti
  double compute_k44 (double lp);
  
  //zatizeni od ucinku rovnomerneho spojiteho zatizeni (prava strana);
  double compute_f4 (double lp);
  
  //matice tuhosti pro stanoveni posouvajici sily a ohyboveho momentu na elementu delky lp
  void compute_kp (double lp);
  
  //prava strana (+zatizeni od ucinku spojiteho zatizeni); pro stanoveni posouvajici sily a ohyboveho momentu na elementu delky lp
  void compute_fkp (double lp);
  
  //matice tuhosti pro stanoveni normalove sily na elementu delky lp
  void compute_kn (double lp);
  
  //prava strana (+zatizeni od ucinku spojiteho zatizeni); pro stanoveni normalove sily na elementu delky lp
  void compute_fkn (double lp);
  
  
  void compute_all (long ipp,double lp);


  void stiffness_matrix (matrix &sm,long ipp,double lp);
   
  void internal_forces (vector &ifor,long ipp,double lp);




  
  //  charakteristicka pevnost betonu v tlaku v [kPa] nap. fck = -25000 kPa
  double fck;
  
  //  pevnost oceli v tahu v [kPa] nap. fy = 490000 kPa
  double fy;
  
  //  pevnost oceli v tahu v [kPa] odpovidajici pretvorevi 0.05 nap. ft= 550000 kPa
  double ft;
  
  //  pretvoreni betonu v tlaku jemuz odpovida max. hodnota napeti fck napr. epsc1 = -0.0022
  double epsc1;
  
  //  mezni pretvoreni betonu v tlaku napr. epscm = -0.0035
  double epscm;
  
  //  mezni pretvoreni oceli v tahu napr. epss1lim = 0.01
  double epss1lim;
  
  //  plocha vyztuze umistene u hornich vlaken prurezu v [m2] napr. ash = 0.0032168 m2 (konvence dle deformacni metody)
  double ash;
  
  //  plocha vyztuze umistene u dolnich vlaken prurezu v [m2] napr. asd = 0.0032168 m2 (konvence dle deformacni metody)
  double asd;
  
  //  kryti vyztuze umistene u hornich vlaken prurezu + polomer vyztuze v [m] napr. ch = 0.041 m (konvence dle deformacni metody)
  double ch;
  
  //  kryti vyztuze umistene u dolnich vlaken prurezu + polomer vyztuze v [m] napr. cd = 0.041 m (konvence dle deformacni metody)
  double cd;
  
  //  modul pruznosti oceli pred plastickym stavem v [kPa] napr. e0c = 200000000 kPa
  double e0c;
  
  //  vyska prurezu v [m] napr. h = 0.6 m
  double h;
  
  //  sirka prurezu v [m] napr. b = 0.3 m
  double b;
  
  // hodnota rovnomerneho spojiteho zatizeni kolmo na strednici prutu v [kN/m] (u pruvlaku kladne hodnoty ve smeru gravitace, u sloupu z leva do prava)
  double fp;
  
  //  hodnota rovnomerneho spojiteho zatizeni ve smeru strednice prutu v [kN/m] (u sloupu kladne hodnoty ve smeru gravitace u pruvlaku z prava do leva)
  double fn;

  

  double fc,ecnom,epsuk,e0cn,k,a,d,as1,as2,z1,z2,z3p,z3,n0,epsn,nmax,an1,an2,in1,in2,kinh,kind,kin;

  double epschr1,mhr1,epschr2,mhr2,epschr3,mhr3,epschr4,epscmax1,epschr4b,epschrk1a,mhr4,mhr4b,epscmax1p,mmax1a,mh1,epsch1;

  double mh1hrk1a,mhrk1a,mh3,epsch3,mh2,epsch2,epscder2,mmax1,mhrk1,epschrk1,mh1hrk1,epschr5,epscs11,mhr5,mmax2,ms11;

  double mh5a,epsch5a,mh5p,epsch5p,mh25a,epscder4,mmax5,mh5,epsch5,epscp,epschr6,mhr6,mmax3,mh6a,epsch6a,mh7,epsch7,epsch12;

  double epss1h1,epss1h2,mh01,mh12,mh25,mh36a,epscder1,mh6,epsch6,mh36,mmax4,mh7maxa,mhmaxa,epscder3,mhmax,mh8p,epsch8p,mh8;

  double epsch8,mh78,mhoc1,mmaxoc1,epss1oc1,mhoc2,epss1oc2,mmaxoc2;

  double mht1,mht2,mht3,mht4,mmax,epscht1,epscht2,epscht3,epscht4,epscmax;

  double *m,*mpp,*tcv;

  double *mp,*tcn,*epscsp,*epss1,*epss1sp,*a1sp,*a2spp,*a2sp,*b1sp,*b2spp,*b2sp,*mr,*epscr,*epss1r,*epss1spp,*xr,*xsp;

  double *ir1,*ims1,*ims2,*ir2,*ir3,*ims3,*ic1,*ic2,*icp,*c1,*c2,*c1p,*c2p,*kih,*kid,*ki;

  double k11,k12,k13,k14,f1,k21,k22,k23,k24,f2,k31,k32,k33,k34,f3,k41,k42,k43,k44,f4;

  
  vector fkn,fkp;

  matrix kp,kn;

  //  pocet iteraci
  long iter;
  
  double zero;
};

#endif
