#ifndef OUTDRIVERM_H
#define OUTDRIVERM_H

#include <stdio.h>
#include "galias.h"
#include "selection.h"
#include "xfile.h"
#include "alias.h"
#include "outdiagm.h"
#include "matrix.h"
#include "siftop.h"

#ifndef FNAMELEN
 #define FNAMELEN 1001
#endif

//class nodeoutm;
//class nodeoutgm;
//class elemoutm;
//class elemoutgm;
//class pointoutm;
//class outdriverm;
struct matrix;


class nodeoutm
{
  public :
   /// constructor
   nodeoutm();
   /// destructor
   ~nodeoutm();
   /// reads data from file
   long read(XFILE *in);
   /// prints description data to file
   void print(FILE *out);
   /// prints values for selected nodes
   void print_out(FILE *out, long lcid);
   /// prints displacements at selected nodes
   void print_disp(FILE *out, long lcid);
   /// prints averaged strains at selected nodes
   void print_stra(FILE *out, long lcid);
   /// prints averaged stresses at selected nodes
   void print_stre(FILE *out, long lcid);
   /// prints averaged other values at selected nodes
   void print_other(FILE *out);
   /// prints reactions at selected nodes
   void print_react(FILE *out);
   /// converts selections given by property id to list or range type
   void conv_sel_prop(siftop *top);

   /// prints nodal values at selected steps
   sel dstep;
   /// selections for load cases 
   sel  sellc;

   /// selection of nodes for displacements
   sel selndisp;
   /// selections for displacements
   sel  *seldisp;

   /// selection of nodes for strains
   sel selnstra;
   /// selections for strains
   sel  *selstra;
   /// indices of local coordinate systems for strains
   long *transtra;

   /// selection of nodes for stresses
   sel selnstre;
   /// selections for stresses
   sel  *selstre;
   /// indices of local coordinate systems for stresses
   long *transtre;

   /// selection of nodes for other values
   sel selnoth;
   /// selections for other values
   sel  *seloth;
   
   /// reactions output indicator
   long react;    
};



class nodeoutgm
{
  public :
   /// constructor
   nodeoutgm ();
   /// destructor
   ~nodeoutgm ();
   /// reads data from file
   long read(XFILE *in);
   /// prints description data to file
   void print(FILE *out);
   /// prints values for selected items to grahics file
   void print_graphics(FILE *out, long lcid, const char *desclcid, graphfmt gf, double *ifor);
   /// prints values for selected items to several grahics files
   void print_graphics(const char *outfn, const char *mode, long lcid, const char *desclcid, graphfmt gf, double *ifor);

   /// prints values of selected strains as scalars to graphics file
   void print_gr_stra_scal(FILE *out, long lcid, const char *desclcid, graphfmt gf);
   /// prints values of selected stresses as scalars to graphics file
   void print_gr_stre_scal(FILE *out, long lcid, const char *desclcid, graphfmt gf);
   /// prints values of selected other items as scalars to graphics file
   void print_gr_oth_scal(FILE *out, long lcid, const char *desclcid, graphfmt gf);

   /// prints values of selected strains as vectors to graphics file
   void print_gr_stra_vec(FILE *out, long lcid, const char *desclcid, graphfmt gf);
   /// prints values of selected stresses as vectors to graphics file
   void print_gr_stre_vec(FILE *out, long lcid, const char *desclcid, graphfmt gf);
   /// prints values of selected other items as vectors to graphics file
   void print_gr_oth_vec(FILE *out, long lcid, const char *desclcid, graphfmt gf);

   /// prints values of selected strains as tensors to graphics file
   void print_gr_stra_mtx(FILE *out, long lcid, const char *desclcid, graphfmt gf);
   /// prints values of selected stresses as tensors to graphics file
   void print_gr_stre_mtx(FILE *out, long lcid, const char *desclcid, graphfmt gf);
   /// prints values of selected other items as tensors to graphics file
   void print_gr_oth_mtx(FILE *out, long lcid, const char *desclcid, graphfmt gf);

   /// prints values of selected strains as scalars to several graphics files
   void print_gr_stra_scal(const char *outfn, const char *mode, long lcid, const char *desclcid);
   /// prints values of selected stresses as scalars to several graphics files
   void print_gr_stre_scal(const char *outfn, const char *mode, long lcid, const char *desclcid);
   /// prints values of selected other items as scalars to several graphics files
   void print_gr_oth_scal(const char *outfn, const char *mode, long lcid, const char *desclcid);

   /// prints values of selected strains as vectors to several graphics files
   void print_gr_stra_vec(const char *outfn, const char *mode, long lcid, const char *desclcid);
   /// prints values of selected stresses as vectors to several graphics files
   void print_gr_stre_vec(const char *outfn, const char *mode, long lcid, const char *desclcid);
   /// prints values of selected other items as vectors to several graphics files
   void print_gr_oth_vec(const char *outfn, const char *mode, long lcid, const char *desclcid);

   /// prints values of selected strains as tensors to several graphics files
   void print_gr_stra_mtx(const char *outfn, const char *mode, long lcid, const char *desclcid);
   /// prints values of selected stresses as tensors to sevral graphics files
   void print_gr_stre_mtx(const char *outfn, const char *mode, long lcid, const char *desclcid);
   /// prints values of selected other items as tensors to several graphics files
   void print_gr_oth_mtx(const char *outfn, const char *mode, long lcid, const char *desclcid);

   /// converts selections given by property id to list or range type
   void conv_sel_prop(siftop *top);

   /// prints nodal values at selected steps
   sel dstep;

   /// selections for load cases 
   sel  sellc;

   /// selection of nodes for displacements
   sel selndisp;
   /// selections for displacements
   sel  *seldisp;

   /// selection of items for strains
   sel selnstra;
   /// selections for strains
   sel *selstra;
   /// indices of local coordinate systems for strains
   long *transtra;

   /// selection of items for stresses
   sel selnstre;
   /// selections for stresses
   sel *selstre;
   /// indices of local coordinate systems for stresses
   long *transtre;

   /// selection of items for other values
   sel selnoth;
   /// selections for other values
   sel  *seloth;
   /// selection of items for forces
   sel selnforce;
   /// selections for components of forces
   sel *selforce;
};



class elemoutm
{
  public :
   /// constructor
   elemoutm();
   /// destructor
   ~elemoutm();
   /// reads data from file
   long read(XFILE *in);
   /// prints description data to file
   void print(FILE *out);
   /// prints values for selected elements
   void print_out(FILE *out, long lcid);
   /// prints strains at selected elements
   void print_stra(FILE *out, long lcid);
   /// prints stresses at selected elements
   void print_stre(FILE *out, long stre);
   /// prints other values at selected elements
   void print_other(FILE *out);
   /// converts selections given by property id to list or range type
   void conv_sel_prop(siftop *top);

   /// prints element values at selected steps
   sel dstep;

   /// selections for load cases 
   sel sellc;

   /// selection of elements for strains
   sel selestra;
   /// selections for strains
   sel *selstra;
   /// indices of local coordinate systems for strains
   long *transtra;

   /// selection of elements for stresses
   sel selestre;
   /// selections for stresses
   sel *selstre;
   /// indices of local coordinate systems for stresses
   long *transtre;

   /// selection of elements for other values
   sel seleoth;
   /// selections for other values
   sel  *seloth;
};



class elemoutgm
{
  public :
   /// constructor
   elemoutgm ();
   /// destructor
   ~elemoutgm ();
   /// reads data from file
   long read(XFILE *in);
   /// prints description data to file
   void print(FILE *out);
   /// prints values for selected items to grahics file
   void print_graphics(FILE *out, long lcid, const char *desclcid, graphfmt gf, long idelem1);
   /// prints values for selected items to to several grahics file
   void print_graphics(const char *outfn, const char *mode, long lcid, const char *desclcid, graphfmt gf, long idelem1);

   /// prints values of selected strains as scalars to graphics file
   void print_gr_stra_scal(FILE *out, long lcid, const char *desclcid, graphfmt gf);
   /// prints values of selected stresses as scalars to graphics file
   void print_gr_stre_scal(FILE *out, long lcid, const char *desclcid, graphfmt gf);
   /// prints values of selected other items as scalars to graphics file
   void print_gr_oth_scal(FILE *out, long lcid, const char *desclcid, graphfmt gf);

   /// prints values of selected strains as vectors to graphics file
   void print_gr_stra_vec(FILE *out, long lcid, const char *desclcid, graphfmt gf);
   /// prints values of selected stresses as vectors to graphics file
   void print_gr_stre_vec(FILE *out, long lcid, const char *desclcid, graphfmt gf);
   /// prints values of selected other items as vectors to graphics file
   void print_gr_oth_vec(FILE *out, long lcid, const char *desclcid, graphfmt gf);

   /// prints values of selected strains as tensors to graphics file
   void print_gr_stra_mtx(FILE *out, long lcid, const char *desclcid, graphfmt gf);
   /// prints values of selected stresses as tensors to graphics file
   void print_gr_stre_mtx(FILE *out, long lcid, const char *desclcid, graphfmt gf);
   /// prints values of selected other items as tensors to graphics file
   void print_gr_oth_mtx(FILE *out, long lcid, const char *desclcid, graphfmt gf);

   /// prints values of selected strains as scalars to several graphics files
   void print_gr_stra_scal(const char *outfn, const char *mode, long lcid, const char *desclcid);
   /// prints values of selected stresses as scalars to several graphics files
   void print_gr_stre_scal(const char *outfn, const char *mode, long lcid, const char *desclcid);
   /// prints values of selected other items as scalars to several graphics files
   void print_gr_oth_scal(const char *outfn, const char *mode, long lcid, const char *desclcid);

   /// prints values of selected strains as vectors to several graphics files
   void print_gr_stra_vec(const char *outfn, const char *mode, long lcid, const char *desclcid);
   /// prints values of selected stresses as vectors to several graphics files
   void print_gr_stre_vec(const char *outfn, const char *mode, long lcid, const char *desclcid);
   /// prints values of selected other items as vectors to several graphics files
   void print_gr_oth_vec(const char *outfn, const char *mode, long lcid, const char *desclcid);

   /// prints values of selected strains as tensors to several graphics files
   void print_gr_stra_mtx(const char *outfn, const char *mode, long lcid, const char *desclcid);
   /// prints values of selected stresses as tensors to sevral graphics files
   void print_gr_stre_mtx(const char *outfn, const char *mode, long lcid, const char *desclcid);
   /// prints values of selected other items as tensors to several graphics files
   void print_gr_oth_mtx(const char *outfn, const char *mode, long lcid, const char *desclcid);

   /// converts selections given by property id to list or range type
   void conv_sel_prop(siftop *top);

   /// prints element values at selected steps
   sel dstep;

   /// selections for load cases 
   sel  sellc;

   /// selection of items for strains
   sel selestra;
   /// selections for strains
   sel *selstra;
   /// indices of local coordinate systems for strains
   long *transtra;

   /// selection of items for stresses
   sel selestre;
   /// selections for stresses
   sel *selstre;
   /// indices of local coordinate systems for stresses
   long *transtre;

   /// selection of items for other values
   sel seleoth;
   /// selections for other values
   sel  *seloth;
   
   /// number of the first element for GiD output (normally should be 1), set by print_graphics function
   long ide1;

   /// number of local coordinate systems used at output procedures
   long nlcs;
   /// array of transformation matrices for individual local coordinate systems
   matrix *lcs;
};



class pointoutm
{
  public :
   /// constructor
   pointoutm();
   /// destructor
   ~pointoutm();
   /// reads data from file
   long read(XFILE *in);
   /// prints description data to file
   void print(FILE *out);
   /// prints values for given user defined point
   void print_out(FILE *out, long lcid);
   /// prints strains at given user defined point
   void print_stra(FILE *out, long lcid);
   /// prints stresses at given user defined point
   void print_stre(FILE *out, long lcid);
   /// prints other values at given user defined point
   void print_other(FILE *out, long pid);

   /// prints UDP values at selected steps
   sel dstep;

   /// number of user defined points
   long npnt;

   /// x-coordinate
   double *ksi;
   /// y-coordinate
   double *eta;
   /// z-coordinate
   double *zeta;

   ///  selection of elements
   sel  selelem;

   /// selections for load cases 
   sel  sellc;

   ///  selections of points
   sel  *selpnt;

   /// selections for strains
   sel  *selstra;
    /// indices of local coordinate systems for strains
   long *transtra;

   /// selections for stresses
   sel  *selstre;
   /// indices of local coordinate systems for stresses
   long *transtre;

   /// selections for other values
   sel  *seloth;
};



class outquantm
{
  public:
   /// constructor
   outquantm();
   /// destructor
   ~outquantm();
   /// reads data from file
   long read(XFILE *in);
   /// prints description data to file
   void print(FILE *out);
   /// converts selections given by property id to list or range type
   void conv_sel_prop(siftop *top);
   /// prints values for selected items to grahics file
   void print_graphics(FILE *out, long lcid, const char *desclcid, graphfmt gf, long idelem1);

   /// name of the required mechanical quantity
   mechquant mqn;
   /// quantity label (if '@' is the first character then automatic label will be generated)
   char qlabel[256];
   /// type of point where the quantity is to be printed out (node/elem ipp/elem UDP)
   nodip tpnt;
   /// selection nodal/element ids where the quantity will be printed out
   sel selid;
   /// selected integration points element
   sel seleip;
   /// number of user defined points on selected elements
   long nudp;
   /// natural coordinates of selected user defined points
   double *ksi, *eta, *zeta;
   /// output format of the given quantity (scalar/vector/tensor2d)
   quantchar qf;
   /// indeces of selected quantity components (displacement comps, strain/stres comps, ...)
   sel seliq;
   /// quantity scaling/tranformation flag
   answertype qstf;
   /// function for scaling/tranformation of the quantity
   gfunct qst;
   /// local coordinate system for vector and tensor2d quantities
   long lcs;

   // auxiliary members not read in the quantity record, read/handled in outdriverm
  
   /// type of the given quantity (scalar/vector/tensor2d) determined in mechmat
   quantchar qt;
   /// pointer to the output file name string
   char *outfname;
   /// pointer to the associated output file of the given quantity
   FILE *outf;
   /// index shift of the first element
   long ide1;
   /** Auxiliary array of quantity values, qvv[i] represents pointer to array of the given vector/tensor2d quantity components 
       at i-th node/element ip/element UDP. In the case of elements ip or UDP, qvv[i][k*ncomp+j] is the value of j-th selected 
       quantity component at k-th element ip or UDP and ncomp is the maximum number of vector/tensor2d components.
   */
   static double **qvv;
   /** Auxiliary array of quantity values, qvv[i] represents value of the selected quantity cpomponent
       at i-th node/element ip/element UDP in the case of scalar output format */
   static double *qvs;
   /// maximum number of points where the quantities will be printed out, it is the first dimension of arrays qvv and qvs
   static long maxnpnt;
   /// maximum number of components of quantities that will be printed out, it is the second dimension of array qvv
   static long maxncmp;
};



class outgrfile
{
  public:
   /// reads data from file
   long read(XFILE *in);
   /// prints description data to file
   void print(FILE *out);
  
   /// output file name
   char outfn[1025];  
   /// format of the given output graphic file
   graphfmt gf;
   /// selection of steps which the quantities will be printed in
   sel selstep;
   /// selection of load cases which the quantities will be printed in
   sel sellc;
   /// number of quantities to be printed out
   long nqnt;
   /// array of records with quantity output desprition
   outquantm *qnt;
   /// pointer to the associated output file of the given quantity
   FILE *outf;
};



class outdriverm
{
  public :
   /// constructor
   outdriverm();
   /// destructor
   ~outdriverm();
   /// reads data from file
   long read(XFILE *in);
   /// prints description data to file
   void print(FILE *out);
   /// prints header for output file
   void print_header(FILE *out);
   /// prints header for new step
   void print_newstep(FILE *out, long lcid, long istep, double time);
   /// prints values to the text file
   void print_out(FILE *out, long lcid, long istep, double time);
   /// forced print of values to the text file
   void print_out_forced(FILE *out, long lcid, long istep, double time);
   /// prints diagrams
   void print_diags(long lcid, double lambda, long istep, double *fi);
   /// forced print of diagrams
   void print_diags_forced(long lcid, double lambda, long istep, double *fi);
   /// prints graphics
   void print_graphics(FILE *out, long lcid, double lambda, long istep, double *fi);
   /// forced print of graphics
   void print_graphics_forced(FILE *out, long lcid, double lambda, long istep, double *fi);
   /// creates output graphics files and their headers for GiD separated format (grfmt_gidsp)
   void create_files_gidsp(const char *mode);
   /// creates output graphics files and their headers for general quantity output (grfmt_quant)
   void create_files_quant(const char *mode, long lcid, long istep, char *dlcid);

   /// prints header, points and used elements to the VTK file
   void print_default_vtk(FILE *fname, long lcid, long istep, double lambda);

   ///prints data to the VTK file
   void write_vtk_unkn(FILE *fname, long lcid, const char *desclcid);
   /// tests the output setting whether it requires some local coordinate system
   long testlcs();
   /// converts selections given by property id to list or range type
   void conv_sel_prop(siftop *top);

   /// output text filename
   char outfn[FNAMELEN];
   /// output diagrams filename
   char outdiagfn[FNAMELEN];
   /// output graphics filename
   char outgrfn[FNAMELEN];
   /// generated output graphic file name for grfmt_gid_sep format
   char outgrfngs[FNAMELEN+50];

   /// text output flag
   flagsw textout;
   /// output text file
   FILE *outf;
   /// output diagram files
   FILE **outdiagf;
   /// output graphics file
   FILE *outgr;

   /// description what will be printed from nodal values
   nodeoutm  no;
   /// description what will be printed from element values
   elemoutm  eo;
   /// description what will be printed for each user def. point
   pointoutm po;

   /// description what will be printed from nodal values for graphics
   nodeoutgm  nog;
   /// description what will be printed from element values for graphics
   elemoutgm  eog;
   

   /// number of local coordinate systems  
   long nlcs;
   /**
       array of transformation matrices of the individual local coordinate systems
       rows of the matrix contains base vectors of the given lcs in the global 
       coordinate system  {e'} = [T]*{e} = lcs[i]*{e}
   */
   gfmatrix *lcs;

   /// format of grahics file - 0 - none, 1 - openDX, 2 - FemCAD, 3 - GiD
   graphfmt gf;
   /// number of cuts exported from 3d mesh to GiD
   long ncut;
   /// number of the first node for GiD output (normally should be 1), set by print_init function
   long idn1;
   /// number of the first element for GiD output (normally should be 1), set by print_init function
   long ide1;

   /// number of output diagrams
   long ndiag;
   /// array with description of printed diagrams
   outdiagm *odiag;
   
   ///number of file sequence in VTK graphic output
  long vtk_num;
  /// number of files for grahic output of given quantities
  long ngfq;
  /// array of file and quantity description for graphic otuptu in the case gf == grfmt_quant
  outgrfile *gfq;
  /** 
     array of indicators for creation of graphic file with quantities: cf[i] = 1 -> create i-th file, cf[i] = j < 0 -> i-th file is the same as -(j+1)-th,
     dimension of cf is ngfq.
  */
  ivector cf;
};

#endif
